from frasco import Feature, action, cached_property
import boto
import os
from tempfile import NamedTemporaryFile


class AwsFeature(Feature):
    name = 'aws'
    ignore_attributes = ['s3_connection']
    defaults = {'upload_bucket': None,
                'upload_filename_prefix': '',
                'upload_acl': 'public-read',
                'upload_async': False,
                'upload_signed_url': False,
                'upload_s3_urls_ttl': 3600}

    @cached_property
    def s3_connection(self):
        return boto.connect_s3(self.options.get('access_key'), self.options.get('secret_key'))

    @action()
    def upload_file_to_s3(self, stream_or_filename, filename, bucket=None, prefix=None,\
                          acl=None, mimetype=None, delete_source=False):
        b = self.s3_connection.get_bucket(bucket or self.options['upload_bucket'])
        prefix = prefix or self.options.get('upload_filename_prefix', '')
        k = b.new_key(prefix + filename)
        acl = acl or self.options['upload_acl']
        headers = None
        if mimetype:
            headers = {'Content-Type': mimetype}
        is_filename = isinstance(stream_or_filename, (str, unicode))
        if is_filename:
            k.set_contents_from_filename(stream_or_filename, headers, policy=acl)
        else:
            k.set_contents_from_string(stream_or_filename.read(), headers, policy=acl)
        if is_filename and delete_source:
            os.remove(stream_or_filename)

    @action(default_option='filename')
    def delete_s3_file(self, filename, bucket=None, prefix=None):
        b = self.s3_connection.get_bucket(bucket or self.options['upload_bucket'])
        prefix = prefix or self.options.get('upload_filename_prefix', '')
        b.delete_key(prefix + filename)


try:
    import upload
except ImportError:
    pass


from frasco_upload.backends import StorageBackend, file_upload_backend
from frasco import current_app


@file_upload_backend
class S3StorageBackend(StorageBackend):
    name = 's3'

    def save(self, file, filename):
        if current_app.features.aws.options['upload_async'] and current_app.features.exists('tasks'):
            tmpname = current_app.features.upload.save_uploaded_file_temporarly(file)
            current_app.features.tasks.enqueue('upload_file_to_s3',
                stream_or_filename=tmpname, filename=filename,
                mimetype=file.mimetype, delete_source=True)
        else:
            current_app.features.aws.upload_file_to_s3(file, filename)

    def url_for(self, filename, **kwargs):
        bucket = current_app.features.aws.options['upload_bucket']
        if current_app.features.aws.options['upload_signed_url']:
            b = current_app.features.aws.s3_connection.get_bucket(bucket)
            k = b.get_key(filename)
            kwargs.setdefault('expires_in',
                current_app.features.aws.options['upload_s3_urls_ttl'])
            return k.generate_url(**kwargs)
        return 'https://%s.s3.amazonaws.com/%s' % (bucket, filename)

    def delete(self, filename):
        if current_app.features.aws.options['upload_async'] and current_app.features.exists('tasks'):
            current_app.features.tasks.enqueue('delete_s3_file', filename=filename)
        else:
            current_app.features.aws.delete_s3_file(filename)